use self::add::AddSeries;
use leptos::*;
use leptos_router::*;
use serde::{Deserialize, Serialize};
#[cfg(feature = "ssr")]
use sqlx::FromRow;
use uuid::Uuid;

pub mod add;
pub mod view;

#[component]
pub fn SeriesList() -> impl IntoView {
    let add_series = expect_context::<Action<AddSeries, Result<(), ServerFnError>>>();
    let delete_series = expect_context::<Action<DeleteSeries, Result<(), ServerFnError>>>();
    let all_series = create_resource(
        move || (add_series.version().get(), delete_series.version().get()),
        |_| async move { get_all().await },
    );
    view! {
        <Suspense fallback=loading>
            { move || all_series.get().map(format_all_series)}
        </Suspense>
        <div>
            <A href="add"
                attr:class="btn btn-primary"
            >
                Add Series
            </A>
        </div>
    }
}

fn format_all_series(
    all_series: Result<Vec<Series>, ServerFnError>,
) -> Result<impl IntoView, ServerFnError> {
    Ok(view! {
        <ErrorBoundary fallback=|errors| {
                view! {
                    <div>
                        {errors.get()
                            .into_iter()
                            .map(|(_, e)| e.to_string())
                            .collect::<Vec<_>>()
                            .join(", ")
                        }
                    </div>
                }
            }
        >
            <ul class="list-group mb-3">
                {
                    all_series.map(|all_series| {
                        Some(all_series.into_iter().map(format_series).collect_view())
                    })
                }
            </ul>
        </ErrorBoundary>
    })
}

fn format_series(series: Series) -> impl IntoView {
    let delete_series = expect_context::<Action<DeleteSeries, Result<(), ServerFnError>>>();
    view! {
        <li class="list-group-item">
            <ActionForm action=delete_series>
                <div class="btn-group" style="float: right;">
                    <A href=format!("view/{}", series.id)
                        attr:class="btn btn-sm btn-secondary"
                        attr:style="float: right"
                    >
                        View
                    </A>
                    <input type="submit"
                        class="btn btn-sm btn-secondary"
                        value="Delete"
                    />
                </div>
                <input type="hidden"
                    name="id"
                    value=series.id.to_string() />
            </ActionForm>
            {series.name}
        </li>
    }
}

pub fn loading() -> impl IntoView {
    view! { <div>Loading...</div> }
}

pub fn format_errors(errors: RwSignal<Errors>) -> impl IntoView {
    view! {
        <div>
            {errors.get()
                .into_iter()
                .map(|(_, e)| e.to_string())
                .collect::<Vec<_>>()
                .join(", ")
            }
        </div>
    }
}

#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
#[cfg_attr(feature = "ssr", derive(FromRow))]
pub struct Series {
    id: Uuid,
    name: String,
    source: String,
    imdb: Option<String>,
    wikipedia: Option<String>,
}

#[server]
pub async fn get_all() -> Result<Vec<Series>, ServerFnError> {
    use crate::db::load_statement;
    use sqlx::{Pool, Postgres};
    use tokio_stream::StreamExt;

    let pool: Pool<Postgres> = expect_context();

    let s = load_statement("select_all_series.sql");
    let mut r = sqlx::query_as::<_, Series>(&s).fetch(&pool);

    let mut found = Vec::new();
    while let Some(series) = r.try_next().await? {
        found.push(series);
    }

    Ok(found)
}

#[server(DeleteSeries)]
pub async fn delete(id: String) -> Result<(), ServerFnError> {
    use crate::db::load_statement;
    use sqlx::{Pool, Postgres};

    let pool: Pool<Postgres> = expect_context();

    let s = load_statement("delete_series.sql");
    sqlx::query(&s)
        .bind(id.parse::<Uuid>()?)
        .execute(&pool)
        .await?;
    Ok(())
}
