use super::{format_errors, Series};
use leptos::*;
use leptos_router::*;
use uuid::Uuid;

#[derive(Params, PartialEq)]
pub struct ViewParams {
    id: Uuid,
}

#[component]
pub fn ViewSeries() -> impl IntoView {
    let ps = use_params::<ViewParams>();
    let id = move || {
        ps.with(|ps| {
            ps.as_ref()
                .map(|ps| ps.id.clone())
                .expect("Failed to parse Id value from query string!")
        })
    };
    let details = create_resource(
        move || id(),
        |id| async move { get_series_details(id).await },
    );
    view! {
        <div class="card p-3">
            <div class="card-heading">
                <A href="/"
                    attr:class="btn btn-close"
                    attr:style="float: right;"
                >
                    <span style="display: none;">x</span>
                </A>
                <h2>View Series</h2>
            </div>
            <div class="card-body">
                <Suspense fallback=loading>
                {details.get().map(format_series_details)}
                </Suspense>
            </div>
        </div>
    }
}

fn loading() -> impl IntoView {
    view! {
        {format_field("Name", view!(<span class="placeholder col-4"></span>))}
        {format_field("Source", view!(<span class="placeholder col-12"></span>))}
        {format_field("IMDB", view!(<span class="placeholder col-12"></span>))}
        {format_field("Wikpedia", view!(<span class="placeholder col-12"></span>))}
    }
}

fn format_series_details(
    details: Result<Series, ServerFnError>,
) -> Result<impl IntoView, ServerFnError> {
    Ok(view! {
        <ErrorBoundary fallback=format_errors>
        {details.map(|details| view! {
                {format_field("Name", details.name)}
                {format_field("Source", details.source)}
                {format_field("IMDB", details.imdb)}
                {format_field("Wikipedia", details.wikipedia)}
            })
        }
        </ErrorBoundary>
    })
}

fn format_field<F: IntoView>(label: &'static str, f: F) -> impl IntoView {
    view! {
        <div class="row">
            <div class="col col-md-2 text-bg-secondary border-bottom border-secondary-subtle">{label}</div>
            <div class="col col-md-auto">{f}</div>
        </div>
    }
}

#[server(GetSeriesDetails)]
pub async fn get_series_details(id: Uuid) -> Result<Series, ServerFnError> {
    use crate::db::load_statement;
    use sqlx::{Pool, Postgres};

    let pool = expect_context::<Pool<Postgres>>();
    let s = load_statement("select_series.sql");
    let r = sqlx::query_as::<_, Series>(&s)
        .persistent(true)
        .bind(id)
        .fetch_one(&pool)
        .await?;

    Ok(r)
}
